@@ -390,24 +390,28 @@ void conv_anytoeuc(gchar *outbuf, gint outlen, const gchar *inbuf)
 
 void conv_anytoutf8(gchar *outbuf, gint outlen, const gchar *inbuf)
 {
+#if HAVE_ICONV
 	gchar *tmpstr = NULL;
+#endif /* HAVE_ICONV */
 
 	switch (conv_guess_ja_encoding(inbuf)) {
+#if HAVE_ICONV
 	case C_ISO_2022_JP:
-		tmpstr = conv_codeset_strdup(inbuf, CS_ISO_2022_JP, CS_UTF_8);
+		tmpstr = conv_iconv_strdup(inbuf, CS_ISO_2022_JP, CS_UTF_8);
 		strncpy2(outbuf, tmpstr, outlen);
 		g_free(tmpstr);
 		break;
 	case C_SHIFT_JIS:
-		tmpstr = conv_codeset_strdup(inbuf, CS_SHIFT_JIS, CS_UTF_8);
+		tmpstr = conv_iconv_strdup(inbuf, CS_SHIFT_JIS, CS_UTF_8);
 		strncpy2(outbuf, tmpstr, outlen);
 		g_free(tmpstr);
 		break;
 	case C_EUC_JP:
-		tmpstr = conv_codeset_strdup(inbuf, CS_EUC_JP, CS_UTF_8);
+		tmpstr = conv_iconv_strdup(inbuf, CS_EUC_JP, CS_UTF_8);
 		strncpy2(outbuf, tmpstr, outlen);
 		g_free(tmpstr);
 		break;
+#endif /* HAVE_ICONV */
 	default:
 		strncpy2(outbuf, inbuf, outlen);
 		break;
@@ -790,6 +794,42 @@ void conv_euctodisp(gchar *outbuf, gint outlen, const gchar *inbuf)
 	conv_unreadable_eucjp(outbuf);
 }
 
+void conv_jistoutf8(gchar *outbuf, gint outlen, const gchar *inbuf)
+{
+#if HAVE_ICONV
+	gchar *tmpstr;
+	tmpstr = conv_iconv_strdup(inbuf, CS_ISO_2022_JP, CS_UTF_8);
+	strncpy2(outbuf, tmpstr, outlen);
+	g_free(tmpstr);
+#else
+	strncpy2(outbuf, inbuf, outlen);
+#endif /* HAVE_ICONV */
+}
+
+void conv_sjistoutf8(gchar *outbuf, gint outlen, const gchar *inbuf)
+{
+#if HAVE_ICONV
+	gchar *tmpstr;
+	tmpstr = conv_iconv_strdup(inbuf, CS_SHIFT_JIS, CS_UTF_8);
+	strncpy2(outbuf, tmpstr, outlen);
+	g_free(tmpstr);
+#else
+	strncpy2(outbuf, inbuf, outlen);
+#endif /* HAVE_ICONV */
+}
+
+void conv_euctoutf8(gchar *outbuf, gint outlen, const gchar *inbuf)
+{
+#if HAVE_ICONV
+	gchar *tmpstr;
+	tmpstr = conv_iconv_strdup(inbuf, CS_EUC_JP, CS_UTF_8);
+	strncpy2(outbuf, tmpstr, outlen);
+	g_free(tmpstr);
+#else
+	strncpy2(outbuf, inbuf, outlen);
+#endif /* HAVE_ICONV */
+}
+
 #warning FIXME_GTK2
 #if 0
 void conv_anytodisp(gchar *outbuf, gint outlen, const gchar *inbuf)
@@ -833,7 +873,7 @@ CodeConverter *conv_code_converter_new(const gchar *charset)
 
 	conv = g_new0(CodeConverter, 1);
 #warning FIXME_GTK2
-	conv->code_conv_func = conv_get_code_conv_func(charset, CS_UTF_8);
+	conv->code_conv_func = conv_get_code_conv_func(charset, NULL);
 	conv->charset_str = g_strdup(charset);
 	conv->charset = conv_get_charset_from_str(charset);
 
@@ -915,23 +955,35 @@ CodeConvFunc conv_get_code_conv_func(const gchar *src_charset_str,
 			return conv_noconv;
 	}
 
-	dest_charset = conv_get_charset_from_str(dest_charset_str);
+	if (!dest_charset_str)
+		dest_charset = C_UTF_8;
+	else
+		dest_charset = conv_get_charset_from_str(dest_charset_str);
 
 	if (dest_charset == C_US_ASCII)
 		return conv_ustodisp;
+#warning FIXME_GTK2
+#if 0
 	else if (dest_charset == C_UTF_8 ||
 		 (dest_charset == C_AUTO &&
 		  conv_get_current_charset() == C_UTF_8))
 		return conv_noconv;
+#endif
 
 	switch (src_charset) {
 	case C_ISO_2022_JP:
 	case C_ISO_2022_JP_2:
+	case C_ISO_2022_JP_3:
 		if (dest_charset == C_AUTO &&
 		    conv_get_current_charset() == C_EUC_JP)
 			code_conv = conv_jistodisp;
 		else if (dest_charset == C_EUC_JP)
 			code_conv = conv_jistoeuc;
+		else if (dest_charset == C_AUTO &&
+			 conv_get_current_charset() == C_UTF_8)
+			code_conv = conv_jistoutf8;
+		else if (dest_charset == C_UTF_8)
+			code_conv = conv_jistoutf8;
 		break;
 	case C_US_ASCII:
 		if (dest_charset == C_AUTO)
@@ -962,14 +1014,25 @@ CodeConvFunc conv_get_code_conv_func(const gchar *src_charset_str,
 			code_conv = conv_sjistodisp;
 		else if (dest_charset == C_EUC_JP)
 			code_conv = conv_sjistoeuc;
+		else if (dest_charset == C_AUTO &&
+		    conv_get_current_charset() == C_UTF_8)
+			code_conv = conv_sjistoutf8;
+		else if (dest_charset == C_UTF_8)
+			code_conv = conv_sjistoutf8;
 		break;
 	case C_EUC_JP:
 		if (dest_charset == C_AUTO &&
 		    conv_get_current_charset() == C_EUC_JP)
 			code_conv = conv_euctodisp;
-		else if (dest_charset == C_ISO_2022_JP ||
-			 dest_charset == C_ISO_2022_JP_2)
+		else if (dest_charset == C_ISO_2022_JP   ||
+			 dest_charset == C_ISO_2022_JP_2 ||
+			 dest_charset == C_ISO_2022_JP_3)
 			code_conv = conv_euctojis;
+		else if (dest_charset == C_AUTO &&
+			 conv_get_current_charset() == C_UTF_8)
+			code_conv = conv_euctoutf8;
+		else if (dest_charset == C_UTF_8)
+			code_conv = conv_euctoutf8;
 		break;
 	default:
 		break;
@@ -1117,6 +1180,7 @@ static const struct {
 	{C_KOI8_U,		CS_KOI8_U},
 	{C_ISO_2022_JP,		CS_ISO_2022_JP},
 	{C_ISO_2022_JP_2,	CS_ISO_2022_JP_2},
+	{C_ISO_2022_JP_3,	CS_ISO_2022_JP_3},
 	{C_EUC_JP,		CS_EUC_JP},
 	{C_EUC_JP,		CS_EUCJP},
 	{C_SHIFT_JIS,		CS_SHIFT_JIS},
@@ -1166,6 +1230,7 @@ static const struct {
 	{"ru_RU"	, C_ISO_8859_5	, C_KOI8_R},
 	{"tg_TJ"	, C_KOI8_T	, C_KOI8_T},
 	{"ru_UA"	, C_KOI8_U	, C_KOI8_U},
+	{"uk_UA.CP1251"	, C_WINDOWS_1251, C_KOI8_U},
 	{"uk_UA"	, C_KOI8_U	, C_KOI8_U},
 
 	{"be_BY"	, C_WINDOWS_1251, C_WINDOWS_1251},
@@ -1511,6 +1576,7 @@ gboolean conv_is_multibyte_encoding(CharSet encoding)
 	case C_EUC_CN:
 	case C_ISO_2022_JP:
 	case C_ISO_2022_JP_2:
+	case C_ISO_2022_JP_3:
 	case C_ISO_2022_KR:
 	case C_ISO_2022_CN:
 	case C_SHIFT_JIS:
