@@ -49,6 +49,7 @@
 #include "alertpanel.h"
 #include "menu.h"
 #include "stock_pixmap.h"
+#include "statusbar.h"
 #include "procmsg.h"
 #include "utils.h"
 #include "gtkutils.h"
@@ -114,15 +115,9 @@ static void folderview_append_folder	 (FolderView	*folderview,
 static void folderview_update_node	 (FolderView	*folderview,
 					  GtkCTreeNode	*node);
 
-static GtkCTreeNode *folderview_find_by_name	(GtkCTree	*ctree,
-						 GtkCTreeNode	*node,
-						 const gchar	*name);
-
 static gint folderview_clist_compare	(GtkCList	*clist,
 					 gconstpointer	 ptr1,
 					 gconstpointer	 ptr2);
-static gint folderview_compare_name	(gconstpointer	 a,
-					 gconstpointer	 b);
 
 /* callback functions */
 static gboolean folderview_button_pressed	(GtkWidget	*ctree,
@@ -172,9 +167,6 @@ static void folderview_remove_mailbox_cb(FolderView	*folderview,
 					 guint		 action,
 					 GtkWidget	*widget);
 
-static void folderview_new_imap_folder_cb(FolderView	*folderview,
-					  guint		 action,
-					  GtkWidget	*widget);
 static void folderview_rm_imap_server_cb (FolderView	*folderview,
 					  guint		 action,
 					  GtkWidget	*widget);
@@ -208,7 +200,7 @@ static void folderview_drag_leave_cb     (GtkWidget        *widget,
 					  guint             time,
 					  FolderView       *folderview);
 static void folderview_drag_received_cb  (GtkWidget        *widget,
-					  GdkDragContext   *drag_context,
+					  GdkDragContext   *context,
 					  gint              x,
 					  gint              y,
 					  GtkSelectionData *data,
@@ -234,9 +226,9 @@ static GtkItemFactoryEntry folderview_mail_popup_entries[] =
 
 static GtkItemFactoryEntry folderview_imap_popup_entries[] =
 {
-	{N_("/Create _new folder..."),	NULL, folderview_new_imap_folder_cb, 0, NULL},
-	{N_("/_Rename folder..."),	NULL, folderview_rename_folder_cb,   0, NULL},
-	{N_("/_Delete folder"),		NULL, folderview_delete_folder_cb,   0, NULL},
+	{N_("/Create _new folder..."),	NULL, folderview_new_folder_cb,    0, NULL},
+	{N_("/_Rename folder..."),	NULL, folderview_rename_folder_cb, 0, NULL},
+	{N_("/_Delete folder"),		NULL, folderview_delete_folder_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/Down_load"),		NULL, folderview_download_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
@@ -385,7 +377,7 @@ FolderView *folderview_create(void)
 	gtk_drag_dest_set(ctree, GTK_DEST_DEFAULT_ALL &
 			  ~GTK_DEST_DEFAULT_HIGHLIGHT,
 			  summary_drag_types, 1,
-			  GDK_ACTION_MOVE);
+			  GDK_ACTION_MOVE | GDK_ACTION_COPY);
 	g_signal_connect(G_OBJECT(ctree), "drag_motion",
 			 G_CALLBACK(folderview_drag_motion_cb),
 			 folderview);
@@ -584,6 +576,50 @@ void folderview_update_msg_num(FolderView *folderview, GtkCTreeNode *row)
 	folderview_update_node(folderview, row);
 }
 
+void folderview_append_item(FolderItem *item)
+{
+	FolderItem *parent;
+	GList *list;
+
+	g_return_if_fail(item != NULL);
+	g_return_if_fail(item->folder != NULL);
+	g_return_if_fail(item->parent != NULL);
+
+	parent = item->parent;
+
+	for (list = folderview_list; list != NULL; list = list->next) {
+		FolderView *folderview = (FolderView *)list->data;
+		GtkCTree *ctree = GTK_CTREE(folderview->ctree);
+		GtkCTreeNode *node, *child;
+
+		node = gtk_ctree_find_by_row_data(ctree, NULL, parent);
+		if (node) {
+			child = gtk_ctree_find_by_row_data(ctree, node, item);
+			if (!child) {
+				gchar *text[N_FOLDER_COLS] =
+					{NULL, "0", "0", "0"};
+
+				gtk_clist_freeze(GTK_CLIST(ctree));
+
+				text[COL_FOLDER] = item->name;
+				child = gtk_ctree_insert_node
+					(ctree, node, NULL, text,
+					 FOLDER_SPACING,
+					 folderxpm, folderxpmmask,
+					 folderopenxpm, folderopenxpmmask,
+					 FALSE, FALSE);
+				gtk_ctree_node_set_row_data(ctree, child, item);
+				gtk_ctree_expand(ctree, node);
+				folderview_update_node(folderview, child);
+				folderview_sort_folders(folderview, node,
+							item->folder);
+
+				gtk_clist_thaw(GTK_CLIST(ctree));
+			}
+		}
+	}
+}
+
 static void folderview_set_folders(FolderView *folderview)
 {
 	GList *list;
@@ -668,6 +704,10 @@ void folderview_rescan_tree(Folder *folder)
 		 _("Yes"), _("No"), NULL);
 	if (avalue != G_ALERTDEFAULT) return;
 
+	if (!FOLDER_IS_LOCAL(folder) &&
+	    !main_window_toggle_online_if_offline(main_window_get()))
+		return;
+
 	inc_lock();
 	window = label_window_create(_("Rebuilding folder tree..."));
 
@@ -678,6 +718,7 @@ void folderview_rescan_tree(Folder *folder)
 
 	folder_write_list();
 	folderview_set_all();
+	statusbar_pop_all();
 
 	gtk_widget_destroy(window);
 	inc_unlock();
@@ -721,6 +762,12 @@ void folderview_check_new(Folder *folder)
 		folderview = (FolderView *)list->data;
 		ctree = GTK_CTREE(folderview->ctree);
 
+		if (folder && !FOLDER_IS_LOCAL(folder)) {
+			if (!main_window_toggle_online_if_offline
+				(folderview->mainwin))
+				return;
+		}
+
 		inc_lock();
 		main_window_lock(folderview->mainwin);
 		gtk_widget_set_sensitive(folderview->ctree, FALSE);
@@ -744,6 +791,7 @@ void folderview_check_new(Folder *folder)
 		gtk_widget_set_sensitive(folderview->ctree, TRUE);
 		main_window_unlock(folderview->mainwin);
 		inc_unlock();
+		statusbar_pop_all();
 	}
 
 	folder_write_list();
@@ -1059,13 +1107,13 @@ void folderview_update_item(FolderItem *item, gboolean update_summary)
 static void folderview_update_item_foreach_func(gpointer key, gpointer val,
 						gpointer data)
 {
-	folderview_update_item((FolderItem *)key, (gboolean)data);
+	folderview_update_item((FolderItem *)key, GPOINTER_TO_INT(data));
 }
 
 void folderview_update_item_foreach(GHashTable *table, gboolean update_summary)
 {
 	g_hash_table_foreach(table, folderview_update_item_foreach_func,
-			     (gpointer)update_summary);
+			     GINT_TO_POINTER(update_summary));
 }
 
 static gboolean folderview_update_all_updated_func(GNode *node, gpointer data)
@@ -1073,8 +1121,11 @@ static gboolean folderview_update_all_updated_func(GNode *node, gpointer data)
 	FolderItem *item;
 
 	item = FOLDER_ITEM(node->data);
-	if (item->updated)
-		folderview_update_item(item, (gboolean)data);
+	if (item->updated) {
+		debug_print("folderview_update_all_updated(): '%s' is updated\n", item->path);
+		folderview_update_item(item, GPOINTER_TO_INT(data));
+	}
+
 	return FALSE;
 }
 
@@ -1087,7 +1138,7 @@ void folderview_update_all_updated(gboolean update_summary)
 		folder = (Folder *)list->data;
 		g_node_traverse(folder->node, G_PRE_ORDER, G_TRAVERSE_ALL, -1,
 				folderview_update_all_updated_func,
-				(gpointer)update_summary);
+				GINT_TO_POINTER(update_summary));
 	}
 }
 
@@ -1197,19 +1248,9 @@ void folderview_new_folder(FolderView *folderview)
 	g_return_if_fail(item != NULL);
 	g_return_if_fail(item->folder != NULL);
 
-	switch (FOLDER_TYPE(item->folder)) {
-	case F_MH:
-	case F_MBOX:
-	case F_MAILDIR:
+	if (FOLDER_IS_LOCAL(item->folder) ||
+	    FOLDER_TYPE(item->folder) == F_IMAP)
 		folderview_new_folder_cb(folderview, 0, NULL);
-		break;
-	case F_IMAP:
-		folderview_new_imap_folder_cb(folderview, 0, NULL);
-		break;
-	case F_NEWS:
-	default:
-		break;
-	}
 }
 
 void folderview_rename_folder(FolderView *folderview)
@@ -1331,8 +1372,11 @@ static gboolean folderview_button_pressed(GtkWidget *ctree, GdkEventButton *even
 			update_tree = remove_tree = TRUE;
 			if (folder->account)
 				folder_property = TRUE;
-		} else
-			search_folder = folder_property = TRUE;
+		} else {
+			folder_property = TRUE;
+			if (folderview->selected == folderview->opened)
+				search_folder = TRUE;
+		}
 		if (FOLDER_IS_LOCAL(folder) || FOLDER_TYPE(folder) == F_IMAP) {
 			if (item->parent == NULL)
 				update_tree = rescan_tree = TRUE;
@@ -1553,29 +1597,6 @@ static void folderview_col_resized(GtkCList *clist, gint column, gint width,
 	}
 }
 
-static GtkCTreeNode *folderview_find_by_name(GtkCTree *ctree,
-					     GtkCTreeNode *node,
-					     const gchar *name)
-{
-	FolderItem *item;
-
-	if (!node)
-		node = GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list);
-	if (!node)
-		return NULL;
-
-	node = GTK_CTREE_ROW(node)->children;
-
-	while (node) {
-		item = gtk_ctree_node_get_row_data(ctree, node);
-		if (!folderview_compare_name(item, name))
-			return node;
-		node = GTK_CTREE_ROW(node)->sibling;
-	}
-
-	return NULL;
-}
-
 static void folderview_download_func(Folder *folder, FolderItem *item,
 				     gpointer data)
 {
@@ -1609,14 +1630,8 @@ static void folderview_download_cb(FolderView *folderview, guint action,
 	g_return_if_fail(item != NULL);
 	g_return_if_fail(item->folder != NULL);
 
-	if (!prefs_common.online_mode) {
-		if (alertpanel(_("Offline"),
-			       _("You are offline. Go online?"),
-			       _("Yes"), _("No"), NULL) == G_ALERTDEFAULT)
-			main_window_toggle_online(folderview->mainwin, TRUE);
-		else
-			return;
-	}
+	if (!main_window_toggle_online_if_offline(folderview->mainwin))
+		return;
 
 	main_window_cursor_wait(mainwin);
 	inc_lock();
@@ -1638,6 +1653,7 @@ static void folderview_download_cb(FolderView *folderview, guint action,
 	main_window_unlock(mainwin);
 	inc_unlock();
 	main_window_cursor_normal(mainwin);
+	statusbar_pop_all();
 }
 
 static void folderview_update_tree_cb(FolderView *folderview, guint action,
@@ -1664,39 +1680,49 @@ static void folderview_new_folder_cb(FolderView *folderview, guint action,
 				     GtkWidget *widget)
 {
 	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
-	gchar *text[N_FOLDER_COLS] = {NULL, "0", "0", "0"};
 	FolderItem *item;
 	FolderItem *new_item;
 	gchar *new_folder;
-	gchar *name, *name_;
-	GtkCTreeNode *node;
+	gchar *name;
+	gchar *p;
 
 	if (!folderview->selected) return;
 
 	item = gtk_ctree_node_get_row_data(ctree, folderview->selected);
 	g_return_if_fail(item != NULL);
 	g_return_if_fail(item->folder != NULL);
+	if (FOLDER_TYPE(item->folder) == F_IMAP)
+		g_return_if_fail(item->folder->account != NULL);
 
-	new_folder = input_dialog(_("New folder"),
-				  _("Input the name of new folder:"),
-				  _("NewFolder"));
+	if (FOLDER_TYPE(item->folder) == F_IMAP) {
+		new_folder = input_dialog
+			(_("New folder"),
+			 _("Input the name of new folder:\n"
+			   "(if you want to create a folder to store subfolders,\n"
+			   " append `/' at the end of the name)"),
+			 _("NewFolder"));
+	} else {
+		new_folder = input_dialog(_("New folder"),
+					  _("Input the name of new folder:"),
+					  _("NewFolder"));
+	}
 	if (!new_folder) return;
+	AUTORELEASE_STR(new_folder, {g_free(new_folder); return;});
 
-	if (strchr(new_folder, G_DIR_SEPARATOR) != NULL) {
+	p = strchr(new_folder, G_DIR_SEPARATOR);
+	if ((p && FOLDER_TYPE(item->folder) != F_IMAP) ||
+	    (p && FOLDER_TYPE(item->folder) == F_IMAP && *(p + 1) != '\0')) {
 		alertpanel_error(_("`%c' can't be included in folder name."),
 				 G_DIR_SEPARATOR);
-		g_free(new_folder);
 		return;
 	}
 
-	name_ = trim_string(new_folder, 32);
-	Xstrdup_a(name, name_, {g_free(new_folder); return;});
-	g_free(name_);
+	name = trim_string(new_folder, 32);
+	AUTORELEASE_STR(name, {g_free(name); return;});
 
 	/* find whether the directory already exists */
-	if (folderview_find_by_name(ctree, folderview->selected, new_folder)) {
+	if (folder_find_child_item_by_name(item, new_folder)) {
 		alertpanel_error(_("The folder `%s' already exists."), name);
-		g_free(new_folder);
 		return;
 	}
 
@@ -1704,25 +1730,10 @@ static void folderview_new_folder_cb(FolderView *folderview, guint action,
 						      new_folder);
 	if (!new_item) {
 		alertpanel_error(_("Can't create the folder `%s'."), name);
-		g_free(new_folder);
 		return;
 	}
-	g_free(new_folder);
-
-	gtk_clist_freeze(GTK_CLIST(ctree));
-
-	text[COL_FOLDER] = new_item->name;
-	node = gtk_ctree_insert_node(ctree, folderview->selected, NULL, text,
-				     FOLDER_SPACING,
-				     folderxpm, folderxpmmask,
-				     folderopenxpm, folderopenxpmmask,
-				     FALSE, FALSE);
-	gtk_ctree_expand(ctree, folderview->selected);
-	gtk_ctree_node_set_row_data(ctree, node, new_item);
-	folderview_sort_folders(folderview, folderview->selected, item->folder);
-
-	gtk_clist_thaw(GTK_CLIST(ctree));
 
+	folderview_append_item(new_item);
 	folder_write_list();
 }
 
@@ -1732,7 +1743,7 @@ static void folderview_rename_folder_cb(FolderView *folderview, guint action,
 	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
 	FolderItem *item;
 	gchar *new_folder;
-	gchar *name, *name_;
+	gchar *name;
 	gchar *message;
 	gchar *old_path;
 	gchar *old_id;
@@ -1745,29 +1756,25 @@ static void folderview_rename_folder_cb(FolderView *folderview, guint action,
 	g_return_if_fail(item->path != NULL);
 	g_return_if_fail(item->folder != NULL);
 
-	name_ = trim_string(item->name, 32);
-	Xstrdup_a(name, name_, return);
-	g_free(name_);
+	name = trim_string(item->name, 32);
 	message = g_strdup_printf(_("Input new name for `%s':"), name);
 	new_folder = input_dialog(_("Rename folder"), message,
 				  g_basename(item->path));
 	g_free(message);
+	g_free(name);
 	if (!new_folder) return;
+	AUTORELEASE_STR(new_folder, {g_free(new_folder); return;});
 
 	if (strchr(new_folder, G_DIR_SEPARATOR) != NULL) {
 		alertpanel_error(_("`%c' can't be included in folder name."),
 				 G_DIR_SEPARATOR);
-		g_free(new_folder);
 		return;
 	}
 
-	if (folderview_find_by_name
-		(ctree, GTK_CTREE_ROW(folderview->selected)->parent,
-		 new_folder)) {
+	if (folder_find_child_item_by_name(item->parent, new_folder)) {
 		name = trim_string(new_folder, 32);
 		alertpanel_error(_("The folder `%s' already exists."), name);
 		g_free(name);
-		g_free(new_folder);
 		return;
 	}
 
@@ -1777,10 +1784,8 @@ static void folderview_rename_folder_cb(FolderView *folderview, guint action,
 	if (item->folder->klass->rename_folder(item->folder, item,
 					       new_folder) < 0) {
 		g_free(old_id);
-		g_free(new_folder);
 		return;
 	}
-	g_free(new_folder);
 
 	if (folder_get_default_folder() == item->folder)
 		prefs_filter_rename_path(old_path, item->path);
@@ -1815,7 +1820,7 @@ static void folderview_delete_folder_cb(FolderView *folderview, guint action,
 	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
 	Folder *folder;
 	FolderItem *item;
-	gchar *message, *name, *name_;
+	gchar *message, *name;
 	AlertValue avalue;
 	gchar *old_path;
 	gchar *old_id;
@@ -1829,9 +1834,8 @@ static void folderview_delete_folder_cb(FolderView *folderview, guint action,
 
 	folder = item->folder;
 
-	name_ = trim_string(item->name, 32);
-	Xstrdup_a(name, name_, return);
-	g_free(name_);
+	name = trim_string(item->name, 32);
+	AUTORELEASE_STR(name, {g_free(name); return;});
 	message = g_strdup_printf
 		(_("All folder(s) and message(s) under `%s' will be deleted.\n"
 		   "Do you really want to delete?"), name);
@@ -1871,7 +1875,7 @@ static void folderview_remove_mailbox_cb(FolderView *folderview, guint action,
 	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
 	GtkCTreeNode *node;
 	FolderItem *item;
-	gchar *name, *name_;
+	gchar *name;
 	gchar *message;
 	AlertValue avalue;
 
@@ -1882,15 +1886,14 @@ static void folderview_remove_mailbox_cb(FolderView *folderview, guint action,
 	g_return_if_fail(item->folder != NULL);
 	if (item->parent) return;
 
-	name_ = trim_string(item->folder->name, 32);
-	Xstrdup_a(name, name_, return);
-	g_free(name_);
+	name = trim_string(item->folder->name, 32);
 	message = g_strdup_printf
 		(_("Really remove the mailbox `%s' ?\n"
 		   "(The messages are NOT deleted from the disk)"), name);
 	avalue = alertpanel(_("Remove mailbox"), message,
 			    _("Yes"), _("+No"), NULL);
 	g_free(message);
+	g_free(name);
 	if (avalue != G_ALERTDEFAULT) return;
 
 	if (folderview->summaryview->folder_item &&
@@ -1903,86 +1906,13 @@ static void folderview_remove_mailbox_cb(FolderView *folderview, guint action,
 	folder_write_list();
 }
 
-static void folderview_new_imap_folder_cb(FolderView *folderview, guint action,
-					  GtkWidget *widget)
-{
-	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
-	gchar *text[N_FOLDER_COLS] = {NULL, "0", "0", "0"};
-	GtkCTreeNode *node;
-	FolderItem *item;
-	FolderItem *new_item;
-	gchar *new_folder;
-	gchar *name, *name_;
-	gchar *p;
-
-	if (!folderview->selected) return;
-
-	item = gtk_ctree_node_get_row_data(ctree, folderview->selected);
-	g_return_if_fail(item != NULL);
-	g_return_if_fail(item->folder != NULL);
-	g_return_if_fail(FOLDER_TYPE(item->folder) == F_IMAP);
-	g_return_if_fail(item->folder->account != NULL);
-
-	new_folder = input_dialog
-		(_("New folder"),
-		 _("Input the name of new folder:\n"
-		   "(if you want to create a folder to store subfolders,\n"
-		   " append `/' at the end of the name)"),
-		 _("NewFolder"));
-	if (!new_folder) return;
-
-	if ((p = strchr(new_folder, G_DIR_SEPARATOR)) != NULL &&
-	    *(p + 1) != '\0') {
-		alertpanel_error(_("`%c' can't be included in folder name."),
-				 G_DIR_SEPARATOR);
-		g_free(new_folder);
-		return;
-	}
-
-	name_ = trim_string(new_folder, 32);
-	Xstrdup_a(name, name_, return);
-	g_free(name_);
-
-	/* find whether the directory already exists */
-	if (folderview_find_by_name(ctree, folderview->selected, new_folder)) {
-		alertpanel_error(_("The folder `%s' already exists."), name);
-		g_free(new_folder);
-		return;
-	}
-
-	new_item = item->folder->klass->create_folder(item->folder, item,
-						      new_folder);
-	if (!new_item) {
-		alertpanel_error(_("Can't create the folder `%s'."), name);
-		g_free(new_folder);
-		return;
-	}
-	g_free(new_folder);
-
-	gtk_clist_freeze(GTK_CLIST(ctree));
-
-	text[COL_FOLDER] = new_item->name;
-	node = gtk_ctree_insert_node(ctree, folderview->selected, NULL, text,
-				     FOLDER_SPACING,
-				     folderxpm, folderxpmmask,
-				     folderopenxpm, folderopenxpmmask,
-				     FALSE, FALSE);
-	gtk_ctree_expand(ctree, folderview->selected);
-	gtk_ctree_node_set_row_data(ctree, node, new_item);
-	folderview_sort_folders(folderview, folderview->selected, item->folder);
-
-	gtk_clist_thaw(GTK_CLIST(ctree));
-
-	folder_write_list();
-}
-
 static void folderview_rm_imap_server_cb(FolderView *folderview, guint action,
 					 GtkWidget *widget)
 {
 	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
 	FolderItem *item;
 	PrefsAccount *account;
-	gchar *name, *name_;
+	gchar *name;
 	gchar *message;
 	AlertValue avalue;
 
@@ -1994,13 +1924,12 @@ static void folderview_rm_imap_server_cb(FolderView *folderview, guint action,
 	g_return_if_fail(FOLDER_TYPE(item->folder) == F_IMAP);
 	g_return_if_fail(item->folder->account != NULL);
 
-	name_ = trim_string(item->folder->name, 32);
-	Xstrdup_a(name, name_, return);
-	g_free(name_);
+	name = trim_string(item->folder->name, 32);
 	message = g_strdup_printf(_("Really delete IMAP4 account `%s'?"), name);
 	avalue = alertpanel(_("Delete IMAP4 account"), message,
 			    _("Yes"), _("+No"), NULL);
 	g_free(message);
+	g_free(name);
 
 	if (avalue != G_ALERTDEFAULT) return;
 
@@ -2023,7 +1952,6 @@ static void folderview_new_news_group_cb(FolderView *folderview, guint action,
 					 GtkWidget *widget)
 {
 	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
-	gchar *text[N_FOLDER_COLS] = {NULL, "0", "0", "0"};
 	GtkCTreeNode *servernode, *node;
 	Folder *folder;
 	FolderItem *item;
@@ -2085,23 +2013,14 @@ static void folderview_new_news_group_cb(FolderView *folderview, guint action,
 	for (cur = new_subscr; cur != NULL; cur = cur->next) {
 		gchar *name = (gchar *)cur->data;
 
-		if (folderview_find_by_name(ctree, servernode, name) != NULL)
+		if (folder_find_child_item_by_name(rootitem, name) != NULL)
 			continue;
 
-		text[COL_FOLDER] = name;
-		node = gtk_ctree_insert_node(ctree, servernode, NULL, text,
-					     FOLDER_SPACING,
-					     folderxpm, folderxpmmask,
-					     folderopenxpm, folderopenxpmmask,
-					     FALSE, FALSE);
-		gtk_ctree_expand(ctree, servernode);
-
 		newitem = folder_item_new(name, name);
 		folder_item_append(rootitem, newitem);
-		gtk_ctree_node_set_row_data(ctree, node, newitem);
+		folderview_append_item(newitem);
 	}
 
-	folderview_sort_folders(folderview, servernode, folder);
 	gtk_clist_thaw(GTK_CLIST(ctree));
 
 	slist_free_strings(new_subscr);
@@ -2115,7 +2034,7 @@ static void folderview_rm_news_group_cb(FolderView *folderview, guint action,
 {
 	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
 	FolderItem *item;
-	gchar *name, *name_;
+	gchar *name;
 	gchar *message;
 	AlertValue avalue;
 
@@ -2127,13 +2046,12 @@ static void folderview_rm_news_group_cb(FolderView *folderview, guint action,
 	g_return_if_fail(FOLDER_TYPE(item->folder) == F_NEWS);
 	g_return_if_fail(item->folder->account != NULL);
 
-	name_ = trim_string(item->path, 32);
-	Xstrdup_a(name, name_, return);
-	g_free(name_);
+	name = trim_string(item->path, 32);
 	message = g_strdup_printf(_("Really delete newsgroup `%s'?"), name);
 	avalue = alertpanel(_("Delete newsgroup"), message,
 			    _("Yes"), _("+No"), NULL);
 	g_free(message);
+	g_free(name);
 	if (avalue != G_ALERTDEFAULT) return;
 
 	if (folderview->opened == folderview->selected) {
@@ -2152,7 +2070,7 @@ static void folderview_rm_news_server_cb(FolderView *folderview, guint action,
 	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
 	FolderItem *item;
 	PrefsAccount *account;
-	gchar *name, *name_;
+	gchar *name;
 	gchar *message;
 	AlertValue avalue;
 
@@ -2164,13 +2082,12 @@ static void folderview_rm_news_server_cb(FolderView *folderview, guint action,
 	g_return_if_fail(FOLDER_TYPE(item->folder) == F_NEWS);
 	g_return_if_fail(item->folder->account != NULL);
 
-	name_ = trim_string(item->folder->name, 32);
-	Xstrdup_a(name, name_, return);
-	g_free(name_);
+	name = trim_string(item->folder->name, 32);
 	message = g_strdup_printf(_("Really delete news account `%s'?"), name);
 	avalue = alertpanel(_("Delete news account"), message,
 			    _("Yes"), _("+No"), NULL);
 	g_free(message);
+	g_free(name);
 
 	if (avalue != G_ALERTDEFAULT) return;
 
@@ -2213,6 +2130,28 @@ static void folderview_property_cb(FolderView *folderview, guint action,
 		prefs_folder_item_open(item);
 }
 
+static void folderview_defer_expand_stop(FolderView *folderview)
+{
+	if (folderview->spring_timer > 0) {
+		gtk_timeout_remove(folderview->spring_timer);
+		folderview->spring_timer = 0;
+	}
+	folderview->spring_node = NULL;
+}
+
+static gint folderview_defer_expand(gpointer data)
+{
+	FolderView *folderview = (FolderView *)data;
+
+	if (folderview->spring_node) {
+		gtk_ctree_expand(GTK_CTREE(folderview->ctree),
+				 folderview->spring_node);
+	}
+	folderview_defer_expand_stop(folderview);
+
+	return FALSE;
+}
+
 static gboolean folderview_drag_motion_cb(GtkWidget      *widget,
 					  GdkDragContext *context,
 					  gint            x,
@@ -2234,6 +2173,17 @@ static gboolean folderview_drag_motion_cb(GtkWidget      *widget,
 			acceptable = FOLDER_ITEM_CAN_ADD(item);
 	}
 
+	if (node != folderview->spring_node) {
+		folderview_defer_expand_stop(folderview);
+		if (node && !GTK_CTREE_ROW(node)->expanded &&
+		    GTK_CTREE_ROW(node)->children) {
+			folderview->spring_timer =
+				gtk_timeout_add(1000, folderview_defer_expand,
+						folderview);
+			folderview->spring_node = node;
+		}
+	}
+
 	if (acceptable) {
 		g_signal_handlers_block_by_func
 			(G_OBJECT(widget),
@@ -2242,7 +2192,14 @@ static gboolean folderview_drag_motion_cb(GtkWidget      *widget,
 		g_signal_handlers_unblock_by_func
 			(G_OBJECT(widget),
 			 G_CALLBACK(folderview_selected), folderview);
-		gdk_drag_status(context, context->suggested_action, time);
+		if ((context->actions & GDK_ACTION_MOVE) != 0)
+			gdk_drag_status(context, GDK_ACTION_MOVE, time);
+		else if ((context->actions & GDK_ACTION_COPY) != 0)
+			gdk_drag_status(context, GDK_ACTION_COPY, time);
+		else if ((context->actions & GDK_ACTION_LINK) != 0)
+			gdk_drag_status(context, GDK_ACTION_LINK, time);
+		else
+			gdk_drag_status(context, 0, time);
 	} else {
 		gtk_ctree_select(GTK_CTREE(widget), folderview->opened);
 		gdk_drag_status(context, 0, time);
@@ -2256,11 +2213,12 @@ static void folderview_drag_leave_cb(GtkWidget      *widget,
 				     guint           time,
 				     FolderView     *folderview)
 {
+	folderview_defer_expand_stop(folderview);
 	gtk_ctree_select(GTK_CTREE(widget), folderview->opened);
 }
 
 static void folderview_drag_received_cb(GtkWidget        *widget,
-					GdkDragContext   *drag_context,
+					GdkDragContext   *context,
 					gint              x,
 					gint              y,
 					GtkSelectionData *data,
@@ -2272,6 +2230,8 @@ static void folderview_drag_received_cb(GtkWidget        *widget,
 	FolderItem *item, *src_item;
 	GtkCTreeNode *node;
 
+	folderview_defer_expand_stop(folderview);
+
 	if (gtk_clist_get_selection_info
 		(GTK_CLIST(widget), x - 24, y - 24, &row, &column) == 0)
 		return;
@@ -2280,16 +2240,16 @@ static void folderview_drag_received_cb(GtkWidget        *widget,
 	item = gtk_ctree_node_get_row_data(GTK_CTREE(widget), node);
 	src_item = folderview->summaryview->folder_item;
 	if (FOLDER_ITEM_CAN_ADD(item) && src_item && src_item != item) {
-		if (FOLDER_TYPE(src_item->folder) !=
-		    FOLDER_TYPE(item->folder) ||
-		    (FOLDER_TYPE(item->folder) == F_IMAP &&
-		     src_item->folder != item->folder))
-			summary_copy_selected_to(folderview->summaryview, item);
-		else
+		if ((context->actions & GDK_ACTION_MOVE) != 0) {
 			summary_move_selected_to(folderview->summaryview, item);
-		gtk_drag_finish(drag_context, TRUE, TRUE, time);
+			gtk_drag_finish(context, TRUE, TRUE, time);
+		} else if ((context->actions & GDK_ACTION_COPY) != 0) {
+			summary_copy_selected_to(folderview->summaryview, item);
+			gtk_drag_finish(context, TRUE, TRUE, time);
+		} else
+			gtk_drag_finish(context, FALSE, FALSE, time);
 	} else
-		gtk_drag_finish(drag_context, FALSE, FALSE, time);
+		gtk_drag_finish(context, FALSE, FALSE, time);
 }
 
 static gint folderview_clist_compare(GtkCList *clist,
@@ -2305,12 +2265,3 @@ static gint folderview_clist_compare(GtkCList *clist,
 
 	return g_strcasecmp(item1->name, item2->name);
 }
-
-static gint folderview_compare_name(gconstpointer a, gconstpointer b)
-{
-	const FolderItem *item = a;
-	const gchar *name = b;
-
-	if (!item->path) return -1;
-	return strcmp2(g_basename(item->path), name);
-}
