@@ -279,10 +279,6 @@ static void show_all_header_cb		(MainWindow	*mainwin,
 					 guint		 action,
 					 GtkWidget	*widget);
 
-static void reedit_cb			(MainWindow	*mainwin,
-					 guint		 action,
-					 GtkWidget	*widget);
-
 static void move_to_cb			(MainWindow	*mainwin,
 					 guint		 action,
 					 GtkWidget	*widget);
@@ -309,6 +305,11 @@ static void mark_as_read_cb		(MainWindow	*mainwin,
 static void mark_all_read_cb		(MainWindow	*mainwin,
 					 guint		 action,
 					 GtkWidget	*widget);
+
+static void reedit_cb			(MainWindow	*mainwin,
+					 guint		 action,
+					 GtkWidget	*widget);
+
 static void add_address_cb		(MainWindow	*mainwin,
 					 guint		 action,
 					 GtkWidget	*widget);
@@ -586,6 +587,8 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	 CODESET_ACTION(C_ISO_8859_5)},
 	{N_("/_View/_Code set/Cyrillic (KOI8-_R)"),
 	 CODESET_ACTION(C_KOI8_R)},
+	{N_("/_View/_Code set/Cyrillic (KOI8-U)"),
+	 CODESET_ACTION(C_KOI8_U)},
 	{N_("/_View/_Code set/Cyrillic (Windows-1251)"),
 	 CODESET_ACTION(C_CP1251)},
 	CODESET_SEPARATOR,
@@ -657,8 +660,6 @@ static GtkItemFactoryEntry mainwin_entries[] =
 						"<shift><control><alt>F", reply_cb, COMPOSE_FORWARD_AS_ATTACH, NULL},
 	{N_("/_Message/Redirec_t"),		NULL, reply_cb, COMPOSE_REDIRECT, NULL},
 	{N_("/_Message/---"),			NULL, NULL, 0, "<Separator>"},
-	{N_("/_Message/Re-_edit"),		NULL, reedit_cb, 0, NULL},
-	{N_("/_Message/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Message/M_ove..."),		"<control>O", move_to_cb, 0, NULL},
 	{N_("/_Message/_Copy..."),		"<shift><control>O", copy_to_cb, 0, NULL},
 	{N_("/_Message/_Delete"),		"<control>D", delete_cb,  0, NULL},
@@ -671,13 +672,18 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	{N_("/_Message/_Mark/Mark as rea_d"),
 						NULL, mark_as_read_cb, 0, NULL},
 	{N_("/_Message/_Mark/Mark all _read"),	NULL, mark_all_read_cb, 0, NULL},
+	{N_("/_Message/---"),			NULL, NULL, 0, "<Separator>"},
+	{N_("/_Message/Re-_edit"),		NULL, reedit_cb, 0, NULL},
 
 	{N_("/_Tools"),				NULL, NULL, 0, "<Branch>"},
 	{N_("/_Tools/_Address book"),		"<shift><control>A", addressbook_open_cb, 0, NULL},
 	{N_("/_Tools/Add sender to address boo_k"),
 						NULL, add_address_cb, 0, NULL},
 	{N_("/_Tools/---"),			NULL, NULL, 0, "<Separator>"},
-	{N_("/_Tools/_Filter messages"),		NULL, filter_cb, 0, NULL},
+	{N_("/_Tools/_Filter all messages in folder"),
+						NULL, filter_cb, 0, NULL},
+	{N_("/_Tools/Filter _selected messages"),
+						NULL, filter_cb, 1, NULL},
 	{N_("/_Tools/_Create filter rule"),	NULL, NULL, 0, "<Branch>"},
 	{N_("/_Tools/_Create filter rule/_Automatically"),
 						NULL, create_filter_cb, FILTER_BY_AUTO, NULL},
@@ -870,6 +876,7 @@ MainWindow *main_window_create(SeparateType type)
 	summaryview->messageview = messageview;
 	summaryview->window      = window;
 
+	messageview->statusbar   = statusbar;
 	messageview->mainwin     = mainwin;
 
 	mainwin->window         = window;
@@ -899,6 +906,8 @@ MainWindow *main_window_create(SeparateType type)
 	mainwin->messageview_cid = gtk_statusbar_get_context_id
 		(GTK_STATUSBAR(statusbar), "Message View");
 
+	messageview->statusbar_cid = mainwin->messageview_cid;
+
 	/* allocate colors for summary view and folder view */
 	summaryview->color_marked.red = summaryview->color_marked.green = 0;
 	summaryview->color_marked.blue = (guint16)65535;
@@ -1196,6 +1205,11 @@ static void main_window_show_cur_account(MainWindow *mainwin)
 	g_free(ac_name);
 }
 
+MainWindow *main_window_get(void)
+{
+	return (MainWindow *)mainwin_list->data;
+}
+
 GtkWidget *main_window_get_folder_window(MainWindow *mainwin)
 {
 	switch (mainwin->type) {
@@ -1438,6 +1452,18 @@ void main_window_toggle_online(MainWindow *mainwin, gboolean online)
 		online_switch_clicked(mainwin->online_switch, mainwin);
 }
 
+gboolean main_window_toggle_online_if_offline(MainWindow *mainwin)
+{
+	if (!prefs_common.online_mode) {
+		if (alertpanel(_("Offline"),
+			       _("You are offline. Go online?"),
+			       _("Yes"), _("No"), NULL) == G_ALERTDEFAULT)
+			main_window_toggle_online(mainwin, TRUE);
+	}
+
+	return prefs_common.online_mode;
+}
+
 void main_window_empty_trash(MainWindow *mainwin, gboolean confirm)
 {
 	GList *list;
@@ -1451,6 +1477,7 @@ void main_window_empty_trash(MainWindow *mainwin, gboolean confirm)
 	}
 
 	procmsg_empty_trash();
+	statusbar_pop_all();
 
 	for (list = folder_get_list(); list != NULL; list = list->next) {
 		Folder *folder;
@@ -1668,18 +1695,19 @@ void main_window_set_menu_sensitive(MainWindow *mainwin)
 		{"/Message/Forward"              , M_HAVE_ACCOUNT|M_TARGET_EXIST},
 		{"/Message/Forward as attachment", M_HAVE_ACCOUNT|M_TARGET_EXIST},
 		{"/Message/Redirect"             , M_HAVE_ACCOUNT|M_SINGLE_TARGET_EXIST},
-		{"/Message/Re-edit"              , M_HAVE_ACCOUNT|M_ALLOW_REEDIT},
 		{"/Message/Move..."              , M_TARGET_EXIST|M_ALLOW_DELETE},
 		{"/Message/Copy..."              , M_TARGET_EXIST|M_EXEC},
 		{"/Message/Delete"               , M_TARGET_EXIST|M_ALLOW_DELETE},
 		{"/Message/Mark"                 , M_TARGET_EXIST},
+		{"/Message/Re-edit"              , M_HAVE_ACCOUNT|M_ALLOW_REEDIT},
 
-		{"/Tools/Add sender to address book", M_SINGLE_TARGET_EXIST},
-		{"/Tools/Filter messages"           , M_MSG_EXIST|M_EXEC},
-		{"/Tools/Create filter rule"        , M_SINGLE_TARGET_EXIST|M_UNLOCKED},
-		{"/Tools/Actions"                   , M_TARGET_EXIST|M_UNLOCKED},
-		{"/Tools/Execute"                   , M_MSG_EXIST|M_EXEC},
-		{"/Tools/Delete duplicated messages", M_MSG_EXIST|M_ALLOW_DELETE},
+		{"/Tools/Add sender to address book"   , M_SINGLE_TARGET_EXIST},
+		{"/Tools/Filter all messages in folder", M_MSG_EXIST|M_EXEC},
+		{"/Tools/Filter selected messages"     , M_TARGET_EXIST|M_EXEC},
+		{"/Tools/Create filter rule"           , M_SINGLE_TARGET_EXIST|M_UNLOCKED},
+		{"/Tools/Actions"                      , M_TARGET_EXIST|M_UNLOCKED},
+		{"/Tools/Execute"                      , M_MSG_EXIST|M_EXEC},
+		{"/Tools/Delete duplicated messages"   , M_MSG_EXIST|M_ALLOW_DELETE},
 
 		{"/Configuration", M_UNLOCKED},
 
@@ -2740,11 +2768,6 @@ static void show_all_header_cb(MainWindow *mainwin, guint action,
 				     GTK_CHECK_MENU_ITEM(widget)->active);
 }
 
-static void reedit_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
-{
-	summary_reedit(mainwin->summaryview);
-}
-
 static void mark_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
 {
 	summary_mark(mainwin->summaryview);
@@ -2773,6 +2796,11 @@ static void mark_all_read_cb(MainWindow *mainwin, guint action,
 	summary_mark_all_read(mainwin->summaryview);
 }
 
+static void reedit_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
+{
+	summary_reedit(mainwin->summaryview);
+}
+
 static void add_address_cb(MainWindow *mainwin, guint action,
 			   GtkWidget *widget)
 {
@@ -2871,7 +2899,7 @@ static void delete_duplicated_cb(MainWindow *mainwin, guint action,
 
 static void filter_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
 {
-	summary_filter(mainwin->summaryview);
+	summary_filter(mainwin->summaryview, (gboolean)action);
 }
 
 static void execute_summary_cb(MainWindow *mainwin, guint action,
