@@ -94,6 +94,9 @@ static gboolean mh_is_msg_changed	(Folder		*folder,
 static gint    mh_close			(Folder		*folder,
 					 FolderItem	*item);
 
+static gint    mh_scan_folder_full	(Folder		*folder,
+					 FolderItem	*item,
+					 gboolean	 count_sum);
 static gint    mh_scan_folder		(Folder		*folder,
 					 FolderItem	*item);
 static gint    mh_scan_tree		(Folder		*folder);
@@ -397,8 +400,12 @@ static gint mh_add_msgs(Folder *folder, FolderItem *dest, GSList *file_list,
 			if (MSG_IS_UNREAD(*fileinfo->flags))
 				dest->unread++;
 		} else {
+			if (dest->unmarked_num == 0)
+				dest->new = 0;
+
 			dest->new++;
 			dest->unread++;
+			dest->unmarked_num++;
 		}
 	}
 
@@ -619,6 +626,7 @@ static gint mh_remove_msg(Folder *folder, FolderItem *item, MsgInfo *msginfo)
 		g_free(file);
 		return -1;
 	}
+	g_free(file);
 
 	item->total--;
 	item->updated = TRUE;
@@ -628,7 +636,9 @@ static gint mh_remove_msg(Folder *folder, FolderItem *item, MsgInfo *msginfo)
 		item->unread--;
 	MSG_SET_TMP_FLAGS(msginfo->flags, MSG_INVALID);
 
-	g_free(file);
+	if (msginfo->msgnum == item->last_num)
+		item->last_num = mh_scan_folder_full(folder, item, FALSE);
+
 	return 0;
 }
 
@@ -669,7 +679,8 @@ static gint mh_close(Folder *folder, FolderItem *item)
 	return 0;
 }
 
-static gint mh_scan_folder(Folder *folder, FolderItem *item)
+static gint mh_scan_folder_full(Folder *folder, FolderItem *item,
+				gboolean count_sum)
 {
 	gchar *path;
 	DIR *dp;
@@ -711,15 +722,18 @@ static gint mh_scan_folder(Folder *folder, FolderItem *item)
 
 	if (n_msg == 0)
 		item->new = item->unread = item->total = 0;
-	else {
+	else if (count_sum) {
 		gint new, unread, total, min, max;
 
 		procmsg_get_mark_sum
 			(item, &new, &unread, &total, &min, &max, 0);
+
 		if (n_msg > total) {
-			new = n_msg - total;
+			item->unmarked_num = new = n_msg - total;
 			unread += n_msg - total;
-		}
+		} else
+			item->unmarked_num = 0;
+
 		item->new = new;
 		item->unread = unread;
 		item->total = n_msg;
@@ -733,6 +747,11 @@ static gint mh_scan_folder(Folder *folder, FolderItem *item)
 	return 0;
 }
 
+static gint mh_scan_folder(Folder *folder, FolderItem *item)
+{
+	return mh_scan_folder_full(folder, item, TRUE);
+}
+
 static gint mh_scan_tree(Folder *folder)
 {
 	FolderItem *item;
