@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2004 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -79,14 +79,6 @@ static struct Filter {
 	GHashTable *hdr_table;
 } filter;
 
-/*
-   parameter name, default value, pointer to the prefs variable, data type,
-   pointer to the widget pointer,
-   pointer to the function for data setting,
-   pointer to the function for widget setting
- */
-
-/* widget creating functions */
 static void prefs_filter_create			(void);
 
 static void prefs_filter_set_dialog		(void);
@@ -95,7 +87,7 @@ static void prefs_filter_set_keyword_entry	(MsgInfo	*msginfo,
 						 const gchar	*key);
 static void prefs_filter_set_list		(void);
 static gint prefs_filter_clist_set_row		(gint	 row);
-static Filter *prefs_filter_dialog_to_filter	(void);
+static FilterRule *prefs_filter_dialog_to_rule	(void);
 
 /* callback functions */
 static void prefs_filter_combo_select_cb(GtkWidget	*widget);
@@ -127,6 +119,7 @@ static gboolean prefs_filter_key_pressed(GtkWidget	*widget,
 					 gpointer	 data);
 static void prefs_filter_close		(void);
 
+
 void prefs_filter_open(MsgInfo *msginfo, const gchar *header, const gchar *key)
 {
 	inc_lock();
@@ -504,7 +497,7 @@ void prefs_filter_read_config(void)
 	gchar *rcpath;
 	FILE *fp;
 	gchar buf[PREFSBUFSIZE];
-	Filter *flt;
+	FilterRule *rule;
 
 	debug_print("Reading filter configuration...\n");
 
@@ -518,10 +511,10 @@ void prefs_filter_read_config(void)
 
 	/* remove all previous filter list */
 	while (prefs_common.fltlist != NULL) {
-		flt = (Filter *)prefs_common.fltlist->data;
-		filter_free(flt);
+		rule = (FilterRule *)prefs_common.fltlist->data;
+		filter_rule_free(rule);
 		prefs_common.fltlist = g_slist_remove(prefs_common.fltlist,
-						      flt);
+						      rule);
 	}
 
 	while (fgets(buf, sizeof(buf), fp) != NULL) {
@@ -535,11 +528,11 @@ void prefs_filter_read_config(void)
 			tmp = g_strdup(buf);
 		}
 		g_strchomp(tmp);
-		flt = filter_read_str(tmp);
+		rule = filter_read_str(tmp);
 
-		if (flt) {
+		if (rule) {
 			prefs_common.fltlist =
-				g_slist_append(prefs_common.fltlist, flt);
+				g_slist_append(prefs_common.fltlist, rule);
 		}
 
 		g_free(tmp);
@@ -564,30 +557,34 @@ void prefs_filter_write_config(void)
 	}
 
 	for (cur = prefs_common.fltlist; cur != NULL; cur = cur->next) {
-		Filter *flt = (Filter *)cur->data;
+		FilterRule *rule = (FilterRule *)cur->data;
 		gchar *fstr;
 		const gchar *src_codeset = CS_UTF_8;
 		const gchar *dest_codeset = conv_get_current_charset_str();
 		gchar *tmp;
 
-		tmp = filter_get_str(flt);
-		fstr = conv_codeset_strdup(tmp, src_codeset, dest_codeset);
-		if (!fstr) {
-			g_warning("Faild to convert character set of filter configuration\n");
-			fstr = g_strdup(tmp);
-		}
+		tmp = filter_get_str(rule);
+		if (tmp) {
+			fstr = conv_codeset_strdup(tmp,
+						   src_codeset,
+						   dest_codeset);
+			if (!fstr) {
+				g_warning("Faild to convert character set of filter configuration\n");
+				fstr = g_strdup(tmp);
+			}
   
-		if (fputs(fstr, pfile->fp) == EOF ||
-		    fputc('\n', pfile->fp) == EOF) {
-			FILE_OP_ERROR(rcpath, "fputs || fputc");
-			prefs_file_close_revert(pfile);
-			g_free(rcpath);
+			if (fputs(fstr, pfile->fp) == EOF ||
+			    fputc('\n', pfile->fp) == EOF) {
+				FILE_OP_ERROR(rcpath, "fputs || fputc");
+				prefs_file_close_revert(pfile);
+				g_free(rcpath);
+				g_free(fstr);
+				g_free(tmp);
+				return;
+			}
 			g_free(fstr);
 			g_free(tmp);
-			return;
 		}
-		g_free(fstr);
- 		g_free(tmp);
 	}
 
 	g_free(rcpath);
@@ -601,29 +598,13 @@ void prefs_filter_write_config(void)
 void prefs_filter_rename_path(const gchar *old_path, const gchar *new_path)
 {
 	GSList *cur;
-	gchar *base;
-	gchar *dest_path;
-	gint oldpathlen;
 
 	g_return_if_fail(old_path != NULL);
 	g_return_if_fail(new_path != NULL);
 
 	for (cur = prefs_common.fltlist; cur != NULL; cur = cur->next) {
-		Filter *flt = (Filter *)cur->data;
-
-		oldpathlen = strlen(old_path);
-		if (flt->dest && !strncmp(old_path, flt->dest, oldpathlen)) {
-			base = flt->dest + oldpathlen;
-			while (*base == G_DIR_SEPARATOR) base++;
-			if (*base == '\0')
-				dest_path = g_strdup(new_path);
-			else
-				dest_path = g_strconcat(new_path,
-							G_DIR_SEPARATOR_S,
-							base, NULL);
-			g_free(flt->dest);
-			flt->dest = dest_path;
-		}
+		FilterRule *rule = (FilterRule *)cur->data;
+		filter_rule_rename_dest_path(rule, old_path, new_path);
 	}
 
 	prefs_filter_write_config();
@@ -637,13 +618,13 @@ void prefs_filter_delete_path(const gchar *path)
 	g_return_if_fail(path != NULL);
 
 	for (cur = prefs_common.fltlist; cur != NULL; cur = next) {
-		Filter *flt = (Filter *)cur->data;
+		FilterRule *rule = (FilterRule *)cur->data;
 		next = cur->next;
 
-		if (flt->dest && !strncmp(path, flt->dest, strlen(path))) {
-			filter_free(flt);
+		if (filter_rule_match_dest_path(rule, path)) {
+			filter_rule_free(rule);
 			prefs_common.fltlist =
-				g_slist_remove(prefs_common.fltlist, flt);
+				g_slist_remove(prefs_common.fltlist, rule);
 		}
 	}
 
@@ -665,12 +646,12 @@ static void prefs_filter_set_dialog(void)
 	gtk_clist_set_row_data(clist, row, NULL);
 
 	for (cur = prefs_common.fltlist; cur != NULL; cur = cur->next) {
-		Filter *flt = (Filter *)cur->data;
+		FilterRule *rule = (FilterRule *)cur->data;
 
-		cond_str[0] = filter_get_str(flt);
+		cond_str[0] = filter_get_str(rule);
 		subst_char(cond_str[0], '\t', ':');
 		row = gtk_clist_append(clist, cond_str);
-		gtk_clist_set_row_data(clist, row, flt);
+		gtk_clist_set_row_data(clist, row, rule);
 
 		g_free(cond_str[0]);
 	}
@@ -739,62 +720,62 @@ static void prefs_filter_set_keyword_entry(MsgInfo *msginfo,
 static void prefs_filter_set_list(void)
 {
 	gint row = 1;
-	Filter *flt;
+	FilterRule *rule;
 
 	g_slist_free(prefs_common.fltlist);
 	prefs_common.fltlist = NULL;
 
-	while ((flt = gtk_clist_get_row_data(GTK_CLIST(filter.cond_clist),
+	while ((rule = gtk_clist_get_row_data(GTK_CLIST(filter.cond_clist),
 		row)) != NULL) {
 		prefs_common.fltlist = g_slist_append(prefs_common.fltlist,
-						      flt);
+						      rule);
 		row++;
 	}
 }
 
-#define GET_ENTRY(entry) \
-	entry_text = gtk_entry_get_text(GTK_ENTRY(entry))
+#define GET_ENTRY(str, entry)					\
+	str = gtk_entry_get_text(GTK_ENTRY(entry))
 
 static gint prefs_filter_clist_set_row(gint row)
 {
 	GtkCList *clist = GTK_CLIST(filter.cond_clist);
-	Filter *flt;
-	const gchar *entry_text;
+	FilterRule *rule;
+	const gchar *text;
 	gchar *cond_str[1];
 
 	g_return_val_if_fail(row != 0, -1);
 
 	if (GTK_WIDGET_IS_SENSITIVE(filter.dest_entry))
-		GET_ENTRY(filter.dest_entry);
+		GET_ENTRY(text, filter.dest_entry);
 	else
-		entry_text = FILTER_NOT_RECEIVE;
-	if (entry_text[0] == '\0') {
+		text = FILTER_NOT_RECEIVE;
+	if (text[0] == '\0') {
 		alertpanel_error(_("Destination is not set."));
 		return -1;
 	}
-	GET_ENTRY(filter.hdr_entry1);
-	if (entry_text[0] == '\0') {
+	GET_ENTRY(text, filter.hdr_entry1);
+	if (text[0] == '\0') {
 		alertpanel_error(_("Header name is not set."));
 		return -1;
 	}
 
-	flt = prefs_filter_dialog_to_filter();
+	rule = prefs_filter_dialog_to_rule();
 
-	cond_str[0] = filter_get_str(flt);
+	cond_str[0] = filter_get_str(rule);
 	subst_char(cond_str[0], '\t', ':');
 
 	if (row < 0)
 		row = gtk_clist_append(clist, cond_str);
 	else {
-		Filter *tmpflt;
+		FilterRule *prev_rule;
 
 		gtk_clist_set_text(clist, row, 0, cond_str[0]);
-		tmpflt = gtk_clist_get_row_data(clist, row);
-		if (tmpflt)
-			filter_free(tmpflt);
+		prev_rule = gtk_clist_get_row_data(clist, row);
+		if (prev_rule)
+			filter_rule_free(prev_rule);
 	}
 
-	gtk_clist_set_row_data(clist, row, flt);
+	gtk_clist_set_row_data(clist, row, rule);
 
 	g_free(cond_str[0]);
 
@@ -803,63 +784,78 @@ static gint prefs_filter_clist_set_row(gint row)
 	return row;
 }
 
-static Filter *prefs_filter_dialog_to_filter(void)
+static FilterRule *prefs_filter_dialog_to_rule(void)
 {
-	Filter *flt;
-	const gchar *entry_text;
+	FilterRule *rule;
+	FilterCond *cond;
+	FilterAction *action;
+	const gchar *text;
+	const gchar *name;
+	const gchar *body;
+	FilterMatchType match_type;
+	FilterMatchFlag match_flag;
 
 	if (GTK_WIDGET_IS_SENSITIVE(filter.dest_entry)) {
-		GET_ENTRY(filter.dest_entry);
-		if (entry_text[0] == '\0')
+		GET_ENTRY(text, filter.dest_entry);
+		if (text[0] == '\0')
 			return NULL;
 	}
 
-	GET_ENTRY(filter.hdr_entry1);
-	if (entry_text[0] == '\0')
+	GET_ENTRY(name, filter.hdr_entry1);
+	if (name[0] == '\0')
 		return NULL;
 
-	flt = g_new0(Filter, 1);
+	rule = g_new0(FilterRule, 1);
 
-	flt->name1 = g_strdup(entry_text);
-
-	GET_ENTRY(filter.key_entry1);
-	if (entry_text[0] != '\0')
-		flt->body1 = g_strdup(entry_text);
-
-	GET_ENTRY(filter.hdr_entry2);
-	if (entry_text[0] != '\0' && strcmp(entry_text, _("(none)")) != 0) {
-		flt->name2 = g_strdup(entry_text);
-
-		GET_ENTRY(filter.key_entry2);
-		if (entry_text[0] != '\0')
-			flt->body2 = g_strdup(entry_text);
-	}
+	GET_ENTRY(body, filter.key_entry1);
 
+	match_type = FLT_CONTAIN;
+	match_flag = 0;
 	if (gtk_toggle_button_get_active
 		(GTK_TOGGLE_BUTTON(filter.regex_chkbtn)))
-		flt->flag1 = flt->flag2 = FLT_REGEX;
+		match_type = FLT_REGEX;
+	else {
+		GET_ENTRY(text, filter.pred_entry1);
+		if (strcmp(text, _("contains")) != 0)
+			match_flag = FLT_NOT_MATCH;
+	}
+	cond = filter_cond_new(name, body, match_type, match_flag);
+	rule->cond_list = g_slist_append(rule->cond_list, cond);
 
-	GET_ENTRY(filter.pred_entry1);
-	if (!strcmp(entry_text, _("contains")))
-		flt->flag1 |= FLT_CONTAIN;
-	GET_ENTRY(filter.pred_entry2);
-	if (!strcmp(entry_text, _("contains")))
-		flt->flag2 |= FLT_CONTAIN;
+	GET_ENTRY(name, filter.hdr_entry2);
+	if (name[0] != '\0' && strcmp(name, _("(none)")) != 0) {
+		GET_ENTRY(body, filter.key_entry2);
 
-	GET_ENTRY(filter.cond_entry);
-	if (!strcmp(entry_text, "and"))
-		flt->cond = FLT_AND;
+		match_type = FLT_CONTAIN;
+		match_flag = 0;
+		if (gtk_toggle_button_get_active
+			(GTK_TOGGLE_BUTTON(filter.regex_chkbtn)))
+			match_type = FLT_REGEX;
+		else {
+			GET_ENTRY(text, filter.pred_entry2);
+			if (strcmp(text, _("contains")) != 0)
+				match_flag = FLT_NOT_MATCH;
+		}
+		cond = filter_cond_new(name, body, match_type, match_flag);
+		rule->cond_list = g_slist_append(rule->cond_list, cond);
+	}
+
+	GET_ENTRY(text, filter.cond_entry);
+	if (!strcmp(text, "and"))
+		rule->bool_op = FLT_AND;
 	else
-		flt->cond = FLT_OR;
+		rule->bool_op = FLT_OR;
 
+	action = g_new0(FilterAction, 1);
 	if (GTK_WIDGET_IS_SENSITIVE(filter.dest_entry)) {
-		GET_ENTRY(filter.dest_entry);
-		flt->dest = g_strdup(entry_text);
-		flt->action = FLT_MOVE;
+		GET_ENTRY(text, filter.dest_entry);
+		action->dest_folder = g_strdup(text);
+		action->type = FLT_ACTION_MOVE;
 	} else
-		flt->action = FLT_NOTRECV;
-
-	return flt;
+		action->type = FLT_ACTION_NOT_RECEIVE;
+	rule->action_list = g_slist_append(rule->action_list, action);
+  
+	return rule;
 }
 
 static void prefs_filter_combo_select_cb(GtkWidget *widget)
@@ -914,7 +910,7 @@ static void prefs_filter_register_cb(void)
 static void prefs_filter_substitute_cb(void)
 {
 	GtkCList *clist = GTK_CLIST(filter.cond_clist);
-	Filter *flt;
+	FilterRule *rule;
 	gint row;
 
 	if (!clist->selection) return;
@@ -922,8 +918,8 @@ static void prefs_filter_substitute_cb(void)
 	row = GPOINTER_TO_INT(clist->selection->data);
 	if (row == 0) return;
 
-	flt = gtk_clist_get_row_data(clist, row);
-	if (!flt) return;
+	rule = gtk_clist_get_row_data(clist, row);
+	if (!rule) return;
 
 	prefs_filter_clist_set_row(row);
 }
@@ -931,7 +927,7 @@ static void prefs_filter_substitute_cb(void)
 static void prefs_filter_delete_cb(void)
 {
 	GtkCList *clist = GTK_CLIST(filter.cond_clist);
-	Filter *flt;
+	FilterRule *rule;
 	gint row;
 
 	if (!clist->selection) return;
@@ -943,10 +939,10 @@ static void prefs_filter_delete_cb(void)
 		       _("Yes"), _("No"), NULL) != G_ALERTDEFAULT)
 		return;
 
-	flt = gtk_clist_get_row_data(clist, row);
-	filter_free(flt);
+	rule = gtk_clist_get_row_data(clist, row);
+	filter_rule_free(rule);
 	gtk_clist_remove(clist, row);
-	prefs_common.fltlist = g_slist_remove(prefs_common.fltlist, flt);
+	prefs_common.fltlist = g_slist_remove(prefs_common.fltlist, rule);
 	if (!clist->selection)
 		gtk_clist_select_row(clist, row - 1, -1);
 }
@@ -1000,51 +996,71 @@ static void prefs_filter_bottom(void)
 }
 
 #define ENTRY_SET_TEXT(entry, str) \
-	gtk_entry_set_text(GTK_ENTRY(entry), str ? str : "")
+	gtk_entry_set_text(GTK_ENTRY(entry), (str) ? (str) : "")
 
 static void prefs_filter_select(GtkCList *clist, gint row, gint column,
 				GdkEvent *event)
 {
-	Filter *flt;
-	Filter default_flt = {"Subject", NULL, NULL, NULL,
-			      FLT_CONTAIN, FLT_CONTAIN, FLT_AND,
-			      NULL, FLT_MOVE};
+	FilterRule *rule;
+	FilterCond *cond;
+	FilterAction *action;
+	FilterRule default_rule = {"Default Rule", NULL, FLT_AND, NULL};
 	gboolean is_regex;
 
-	default_flt.name2 = _("(none)");
-
-	flt = gtk_clist_get_row_data(clist, row);
-	if (!flt)
-		flt = &default_flt;
-
-	ENTRY_SET_TEXT(filter.dest_entry, flt->dest);
-	ENTRY_SET_TEXT(filter.hdr_entry1, flt->name1);
-	ENTRY_SET_TEXT(filter.key_entry1, flt->body1);
-	ENTRY_SET_TEXT(filter.hdr_entry2,
-		       flt->name2 ? flt->name2 : _("(none)"));
-	ENTRY_SET_TEXT(filter.key_entry2, flt->body2);
+	rule = gtk_clist_get_row_data(clist, row);
+	if (!rule) {
+		if (!default_rule.cond_list) {
+			cond = filter_cond_new
+				("Subject", NULL, FLT_CONTAIN, 0);
+			default_rule.cond_list = g_slist_append(NULL, cond);
+		}
+		rule = &default_rule;
+	}
 
+	cond = (FilterCond *)rule->cond_list->data;
+	ENTRY_SET_TEXT(filter.hdr_entry1, cond->header);
+	ENTRY_SET_TEXT(filter.key_entry1, cond->body);
 	ENTRY_SET_TEXT(filter.pred_entry1,
-		       FLT_IS_CONTAIN(flt->flag1)
-		       ? _("contains") : _("not contain"));
-	ENTRY_SET_TEXT(filter.pred_entry2,
-		       FLT_IS_CONTAIN(flt->flag2)
-		       ? _("contains") : _("not contain"));
+		       FLT_IS_NOT_MATCH(cond->match_flag)
+		       ? _("not contain") : _("contains"));
+	is_regex = (cond->match_type == FLT_REGEX);
+
+	if (rule->cond_list->next) {
+		cond = (FilterCond *)rule->cond_list->next->data;
+		ENTRY_SET_TEXT(filter.hdr_entry2,
+			       cond->header ? cond->header : _("(none)"));
+		ENTRY_SET_TEXT(filter.key_entry2, cond->body);
+		ENTRY_SET_TEXT(filter.pred_entry2,
+			       FLT_IS_NOT_MATCH(cond->match_flag)
+			       ? _("not contain") : _("contains"));
+	} else {
+		ENTRY_SET_TEXT(filter.hdr_entry2, _("(none)"));
+		ENTRY_SET_TEXT(filter.key_entry2, NULL);
+		ENTRY_SET_TEXT(filter.pred_entry2, _("contains"));
+	}
 
-	is_regex = FLT_IS_REGEX(flt->flag1);
 	gtk_widget_set_sensitive(filter.pred_combo1, !is_regex);
 	gtk_widget_set_sensitive(filter.pred_combo2, !is_regex);
 	gtk_toggle_button_set_active
 		(GTK_TOGGLE_BUTTON(filter.regex_chkbtn), is_regex);
 
 	gtk_entry_set_text(GTK_ENTRY(filter.cond_entry),
-			   flt->cond == FLT_OR ? "or" : "and");
-	if (flt->action == FLT_NOTRECV)
-		gtk_toggle_button_set_active
-			(GTK_TOGGLE_BUTTON(filter.notrecv_radiobtn), TRUE);
-	else
+			   rule->bool_op == FLT_OR ? "or" : "and");
+
+	action = filter_rule_get_last_action(rule);
+	if (action) {
+		ENTRY_SET_TEXT(filter.dest_entry, action->dest_folder);
+		if (action->type == FLT_ACTION_NOT_RECEIVE)
+			gtk_toggle_button_set_active
+				(GTK_TOGGLE_BUTTON(filter.notrecv_radiobtn), TRUE);
+		else
+			gtk_toggle_button_set_active
+				(GTK_TOGGLE_BUTTON(filter.dest_radiobtn), TRUE);
+	} else {
+		ENTRY_SET_TEXT(filter.dest_entry, NULL);
 		gtk_toggle_button_set_active
 			(GTK_TOGGLE_BUTTON(filter.dest_radiobtn), TRUE);
+	}
 }
 
 static void prefs_filter_row_move(GtkCList *clist, gint source_row,
@@ -1094,29 +1110,30 @@ static gboolean prefs_filter_key_pressed(GtkWidget *widget, GdkEventKey *event,
 
 static void prefs_filter_close(void)
 {
-	Filter *flt = NULL, *edit_flt = NULL;
+	FilterRule *rule = NULL, *edit_rule = NULL;
 	gint row;
 	AlertValue aval;
 
 	if (GTK_CLIST(filter.cond_clist)->selection) {
 		row = GPOINTER_TO_INT
 			(GTK_CLIST(filter.cond_clist)->selection->data);
-		flt = gtk_clist_get_row_data(GTK_CLIST(filter.cond_clist), row);
-		edit_flt = prefs_filter_dialog_to_filter();
+		rule = gtk_clist_get_row_data(GTK_CLIST(filter.cond_clist),
+					      row);
+		edit_rule = prefs_filter_dialog_to_rule();
 	}
 
-	if ((!flt && edit_flt) ||
-	    (flt && edit_flt && filter_is_equal(flt, edit_flt) == FALSE)) {
+	if ((!rule && edit_rule) ||
+	    (rule && edit_rule && filter_rule_is_equal(rule, edit_rule) == FALSE)) {
 		aval = alertpanel(_("Entry not saved"),
 				  _("The entry was not saved. Close anyway?"),
 				  _("Yes"), _("No"), NULL);
 		if (G_ALERTDEFAULT != aval) {
-			filter_free(edit_flt);
+			filter_rule_free(edit_rule);
 			return;
 		}
 	}
 
-	filter_free(edit_flt);
+	filter_rule_free(edit_rule);
 
 	if (filter.hdr_table) {
 		g_hash_table_destroy(filter.hdr_table);
