@@ -694,7 +694,7 @@ FILE *procmime_get_text_content(MimeInfo *mimeinfo, FILE *infp)
 	if (mimeinfo->mime_type == MIME_TEXT) {
 		while (fgets(buf, sizeof(buf), tmpfp) != NULL) {
 #warning FIXME_GTK2
-			str = conv_codeset_strdup(buf, src_codeset, CS_UTF_8);
+			str = conv_codeset_strdup(buf, src_codeset, NULL);
 			if (str) {
 				fputs(str, outfp);
 				g_free(str);
